//
//          CMVSource 1.0.0 for AviSynth 2.5.x
//
//  Loads Dwarf Fortress CMV and CCMV movies. Latest release hosted at
//  http://www.gyroshot.com/cmvsource.htm
//
//          Copyright 2011 Robert Martens  robert.martens@gmail.com
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.



#include "DfUtils.h"

#include <fstream> // ifstream
#include <sstream> // stringstream
#include <string>
#include <vector>

#include "avisynth.h"



std::vector<int> LoadColors(std::string filename, IScriptEnvironment* env)
{
  
  std::ifstream colorsIn(filename);

  if (!colorsIn)
    env->ThrowError("CMVSource: Couldn't open color file '%s'", filename);

  std::string colorsBuf;

  while (colorsIn.good())
    colorsBuf.push_back(static_cast<char>(colorsIn.get()));

  colorsIn.close();
  
  // I don't know if this is how Toady validates colors.txt, but since I'm
  // already assuming only sixteen colors, with three components each, this is a
  // quick, easy way to ensure the contents are acceptable.
  int lefts = 0, colons = 0, rights = 0;
  for (unsigned int i = 0; i < colorsBuf.size(); ++i) {
    if (colorsBuf[i] == '[')
      ++lefts;
    else if (colorsBuf[i] == ':')
      ++colons;
    else if (colorsBuf[i] == ']')
      ++rights;
  }

  if ( !(lefts == 48 && colons == 48 && rights == 48) )
    env->ThrowError("CMVSource: Invalid color data in '%s'", filename);

  std::vector<int> components;
  
  for ( unsigned int entry = colorsBuf.find('['), entryLen = 0;
        entry < colorsBuf.size();
        entry += entryLen ) {
  
    std::size_t entryPos = colorsBuf.find('[', entry);
    entryPos = colorsBuf.find('[', entry);
    entryLen = colorsBuf.find('[', entryPos + 1) - entryPos;

    std::size_t tokenPos = entryPos + 1;
    std::size_t tokenLen = colorsBuf.find(':', tokenPos) - tokenPos;
    std::string token = colorsBuf.substr(tokenPos, tokenLen);
  
    int val;
    GetTokenValue(token, colorsBuf) >> val;
    components.push_back(val);
  
  }

  // Inflexible, but simple and reliable given the current format of DF's
  // colors.txt. Sixteen colors, the first eight dull, the second eight bright,
  // to be used with the tile attributes' foreground intensity bit.
  std::vector<int> colors;

  for (int i = 0; i < 16; ++i) {
    int r = components[i*3],
        g = components[i*3+1],
        b = components[i*3+2];
    int color = (r << 16) | (g << 8) | b;
    colors.push_back(color);
  }

  return colors;

}



std::stringstream GetTokenValue(std::string token, std::string buf)
{

  if ( *token.rbegin() != ':' )
    token.push_back(':');

  int fontPos = buf.find(token),
      fontEnd = buf.find(']', fontPos),
      fontLen = fontEnd - (fontPos + token.length());

  std::stringstream value;
  value << buf.substr(fontPos + token.length(), fontLen);
  return value;

}